﻿using System;
using System.Collections.Generic;
using System.IdentityModel.Selectors;
using System.IdentityModel.Tokens;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Security;
using System.ServiceModel.Security.Tokens;
using System.Text;
using System.Threading.Tasks;
using SAMLAuthDemo.EventCollector;
using SamlTools;

namespace SAMLAuthDemo
{
    class Adapter
    {

        public static GenericResult DoUncorrelatedECQuery(String query)
        {
            string serviceAddress = @"http://localhost:8080/amxbpm/EventCollectorQueryService";



            X509Store store = new X509Store(StoreName.TrustedPeople, StoreLocation.CurrentUser);
            store.Open(OpenFlags.ReadOnly);
            X509Certificate2Collection certs = store.Certificates.Find(X509FindType.FindBySubjectName, "SAML", false);
            store.Close();

            SecurityToken token = Adapter.GenerateSAMLToken();

            EventCollectorQueryService svc = Adapter.ConfigureClientProxy(serviceAddress, token,certs[0]);

            
            executeGenericQueryRequest request = new executeGenericQueryRequest();



            request.executeGenericQueryRequest1 = new executeQueryObject();
             Query q = new Query();
            q.correlateSpecified = true;
            q.correlate = false;
            q.filter = query;
            q.requireAllAttributesSpecified = true;
            q.requireAllAttributes = false;

            q.requiredAttribute = new String[] { "messageId", "creationTime", "message","severity" };
            
            request.executeGenericQueryRequest1.Query = q;

            request.executeGenericQueryRequest1.QueryOptions = new QueryOptions();

            request.executeGenericQueryRequest1.QueryOptions.getTotalCountSpecified = true;
            request.executeGenericQueryRequest1.QueryOptions.getTotalCount = true;


            executeGenericQueryResponse response = svc.executeGenericQuery(request);
            
            return response.executeGenericQueryResponse1.GenericResult;
        }

       
        private static EventCollectorQueryService ConfigureClientProxy(string serviceAddress, SecurityToken token, X509Certificate2 certificate)
        {
            ChannelFactory<EventCollectorQueryService> cf = new ChannelFactory<EventCollectorQueryService>(new SamlBinding(),
                new EndpointAddress(new Uri(serviceAddress), EndpointIdentity.CreateX509CertificateIdentity(certificate)));

            // Set the certificate we will use to sign the message
            cf.Credentials.ClientCertificate.Certificate = certificate;
            cf.Endpoint.Contract.ProtectionLevel = System.Net.Security.ProtectionLevel.Sign;


            return cf.CreateChannelWithIssuedToken(token);
        }
        private static SecurityToken GenerateSAMLToken()
        {
            SamlClientCredentials creds = new SamlClientCredentials();

            SecurityTokenManager tokenManager = creds.CreateSecurityTokenManager();

            IssuedSecurityTokenParameters tokenParams = new IssuedSecurityTokenParameters();

            SecurityBindingElement sbe = SecurityBindingElement.CreateIssuedTokenBindingElement(tokenParams);

            SecurityTokenRequirement tokenRequirement = new SecurityTokenRequirement();
            tokenRequirement.TokenType = SecurityTokenTypes.Saml;
            tokenRequirement.KeyType = SecurityKeyType.AsymmetricKey;
            tokenRequirement.Properties.Add("http://schemas.microsoft.com/ws/2006/05/servicemodel/securitytokenrequirement/SecurityBindingElement", sbe);

            SecurityTokenProvider tokenProvider = tokenManager.CreateSecurityTokenProvider(tokenRequirement);

            return tokenProvider.GetToken(TimeSpan.FromDays(20));
        }


    }
}
