using System;

using System.IdentityModel.Selectors;
using System.IdentityModel.Tokens;

using System.IO;

using System.ServiceModel.Security;

using System.Xml;

namespace SamlTools
{
    public class SamlSecurityTokenProvider : SecurityTokenProvider
    {
        SamlAssertion assertion;

        SecurityToken proofToken;

        public SamlSecurityTokenProvider(SamlAssertion assertion, SecurityToken proofToken)
        {
            this.assertion = assertion;
            this.proofToken = proofToken;
        }

        protected override SecurityToken GetTokenCore(TimeSpan timeout)
        {
            SamlSecurityToken samlToken = new SamlSecurityToken(assertion);

            WSSecurityTokenSerializer ser = new WSSecurityTokenSerializer();

            MemoryStream s = new MemoryStream(UInt16.MaxValue);

            XmlWriter xw = XmlWriter.Create(s);

            ser.WriteToken(xw, samlToken);

            s.Seek(0, SeekOrigin.Begin);

            XmlDocument dom = new XmlDocument();
            dom.Load(s);

            SamlAssertionKeyIdentifierClause samlKeyIdentifierClause = samlToken.CreateKeyIdentifierClause<SamlAssertionKeyIdentifierClause>();

            return new GenericXmlSecurityToken(dom.DocumentElement, proofToken, assertion.Conditions.NotBefore, assertion.Conditions.NotOnOrAfter, samlKeyIdentifierClause, samlKeyIdentifierClause, null);
        }
    }
}
